#!/usr/bin/perl -T -I. 
=pod

=head1 NAME

disks_cgi.pl - Handle all disk related web pages

=head1 SYNOPSIS

    <FORM ACTION="disks_cgi.pl">

=head1 DESCRIPTION

The I<disks_cgi.pl> handles the disk database.  It first displays
a report screen.  The user can then select a entry and modify 
or delete it. 

He can also press another button and create a disk.

=head1 FILES

=over 4

=item disks.db

The name of the database file.  The actual path to this file is 
controlled by the I<disks_config.pm> package.

=back

=head1 AUTHOR

Steve Oualline, E<lt>oualline@www.oualline.comE<gt>.

=head1 COPYRIGHT

This program is distributed under the GPL.

=cut
use strict;
use warnings;

use CGI::Thin;
use CGI::Thin::Cookies;
use CGI::Carp;
use POSIX;
use HTML::Entities;

use disks_config;
use disks;

# The data from the form
my %cgi_data = Parse_CGI();

########################################################
# debug -- print debugging information
########################################################
sub debug()
{
    if (-f "/tmp/debug.disk") {
	foreach my $cur_key (sort keys %cgi_data) {
	    print encode_entities($cur_key), " =&gt; ",
	          encode_entities($cgi_data{$cur_key}),
		  "<BR>\n";
	}
    }
}
########################################################
# print_category($current, %name_to_cat) 
#
# Print a catigory option list.
########################################################
sub print_category($%)
{
    my $selected = shift;	# Currently selected category
    my %names = @_;		# The hash of the possibles

    print "    <SELECT NAME=\"CATEGORY\">\n";

    # Loop over each and print it.
    foreach my $cur_cat (sort keys %names) {
	# Check to see if we are selected
	if ($cur_cat eq $selected) {
	    print "        <OPTION VALUE=\"$cur_cat\" SELECTED>",
	    "$names{$cur_cat}</option>\n";
	} else {
	    print "        <OPTION VALUE=\"$cur_cat\">",
	    "$names{$cur_cat}</OPTION>\n";
	}
    }
    print "    </SELECT>\n";
}
########################################################
# display_form($user_cat, $msg) -- Display the 
#	report form with the selected catigory as
#	the default.
########################################################
sub display_form($$)
{
    my $user_cat = shift;	# The user selected category
    my $msg = shift;		# Message for the top of the screen

    # The database for rading the current data
    my $db = disks->new($db_file, 1);
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Disk Database</title>
</head>

<body bgcolor="#FFFFFF">
EOF
	if (defined($msg)) {
	    print "<H1>$msg</H1><P>\n";
	    print "<HR>\n";
	}
    print <<EOF;
<h2>Disk Data</h2>
<form method="post" action="disks_cgi.pl" name="disk">
<p>
EOF
    print_category($user_cat, %category_to_name, "All" => "All");
    print <<EOF;
<INPUT TYPE="SUBMIT" NAME="SUBMIT" VALUE="Update List">
</P>
<P ALIGN="LEFT"> 
    <SELECT NAME="DISKS" SIZE="7">
EOF

    # Loop through each item in the database
    foreach my $cur_id (sort $db->get_keys()) {
        # Filter on category
	my $cur_entry = $db->get_disk($cur_id);

	if (not defined($cur_entry->{CATEGORY})) {
	    $cur_entry->{CATEGORY} = "X";
	}
	# Filter on category
	if (($user_cat ne "All") and 
		($user_cat ne $cur_entry->{CATEGORY})) {
	    next;
	}

	# The description as a nice thing for the CGI screen
	my $cur_desc = encode_entities(
		sprintf "%-3s: %-s", $cur_id, $cur_entry->{DESCRIPTION});

	print "<option value=\"$cur_id\">$cur_desc</option>\n";
    }

    print <<EOF;
    </select>
</p>
<p align="left"> 
<input type="SUBMIT" name="SUBMIT" value="Add">
<input type="SUBMIT" name="SUBMIT" value="Remove">
<input type="SUBMIT" name="SUBMIT" value="Modify">
</p>
EOF
    debug();
    print <<EOF;
</form>
</body>
</html>
EOF
    exit (0);
}


########################################################
# add_modify_form($$) -- Display a add or modify
#	page.
#
# $category -- The catigory selected
# $modify_id -- True if this is a modification request
########################################################
sub add_modify_form($$)
{
    my $category = shift;	# Current category
    my $modify_id = shift;	# Add or modify

    my $add_mod = "Add";	# Add or modify?
    my $description = "";	# Description of the disk

    if (defined($modify_id)) {
        my $db = disks->new($db_file, 1);
        $add_mod = "Modify";

	# Data for this disk
	my $data = $db->get_disk($modify_id);

        $description = $data->{DESCRIPTION};
	if (not defined($description)) {
	    $description = "";
	}
    }
    print <<EOF;
Content-type: text/html

<html>
<head>
<title>Disk Database</title>
</head>

<body bgcolor="#FFFFFF">
<h1>$add_mod Disk</h1>
<form method="post" action="disks_cgi.pl" name="disk">
<p align="left">Category 
EOF

    print_category($category, %category_to_name);

    my $enc_description = encode_entities($description);

    if (defined($modify_id)) {
        print "<INPUT TYPE=\"HIDDEN\" ",
		"NAME=\"ID\" VALUE=\"$modify_id\">\n";
    }
    print <<EOF;
    </p>
    <p align="left"> Description 
      <input type="text" name="DESCRIPTION" VALUE="$enc_description">
    </p>
    <p align="left"> 
      <input type="submit" name="SUBMIT" value="$add_mod Disk">
    </p>
EOF
    debug();
    print <<EOF;
  </form>
</body>
</html>
EOF
    exit (0);
}

# Cookie information
my %cookies = Parse_Cookies();

# The weight from the cookie
my $category = $cookies{CATEGORY};
if (not defined ($category)) {
    $category = "All";
}
if (defined($cgi_data{CATEGORY})) {
    $category = $cgi_data{CATEGORY};
}

# The cookie to send to the user
my $cookie;
$cookie = Set_Cookie(
    NAME => "CATEGORY",   # Cookie's name
    VALUE => $category,   # Value for the cookie
    EXPIRE => "+2d",      # Keep cookie for 2 days
);
print "$cookie";


# Check to see if we should display the form or 
# record the data
if (not defined($cgi_data{SUBMIT})) {
    # Display the form
    display_form($category, undef);
}

#-------------------------------------------------------
if ($cgi_data{SUBMIT} eq "Update List") {
    # Display the form
    display_form($category, undef);
} elsif ($cgi_data{SUBMIT} eq "Add") {
    add_modify_form($category, undef);
#-------------------------------------------------------
} elsif ($cgi_data{SUBMIT} eq "Remove") {
    if (not defined($cgi_data{DISKS})) {
        display_form($category, "ERROR: No disk selected");
    }
    # Id of the disk to remove
    my $id = $cgi_data{DISKS};

    # Open a db to add a disk to
    my $db = disks->new($db_file, 0);	
    $db->delete_disk($id);
    $db = undef;	# Close database

    display_form($category, "Disk $id removed");
#-------------------------------------------------------
} elsif ($cgi_data{SUBMIT} eq "Modify") {
    if (not defined($cgi_data{DISKS})) {
        display_form($category, "ERROR: No disk selected");
    }
    # Id of the disk to modify
    my $id = $cgi_data{DISKS};

    add_modify_form($category, $id);
#-------------------------------------------------------
} elsif ($cgi_data{SUBMIT} eq "Add Disk") {
    # Open a db to add a disk to
    my $db = disks->new($db_file, 0);	

    # Add the disk
    my $id = $db->new_disk($category, 
            DESCRIPTION => $cgi_data{DESCRIPTION},
            CATEGORY => $cgi_data{CATEGORY});

    $db = undef;	# Close database

    # Tell the user the results
    if (defined($id)) {
        display_form($category, "Disk $id added");
    } else {
        display_form($category, "Error: Could not add disk");
    }
#-------------------------------------------------------
} elsif ($cgi_data{SUBMIT} eq "Modify Disk") {
    my $db = disks->new($db_file, 1);	
    my $id = $cgi_data{"ID"};
    $id = $db->modify_disk($id,
            DESCRIPTION => $cgi_data{DESCRIPTION},
            CATEGORY => $cgi_data{CATEGORY});
    if (defined($id)) {
        display_form($category, "Disk $id modified");
    } else {
        display_form($category, "Error: Could not modify disk");
    }
} else {
    # This should never happen.  But if it does
    display_form($category, undef);
}

