=pod

=head1 NAME

disks_gui.pl - Display and modify the disk database

=head1 SYNOPSIS

    perl disks_gui.pl

=head1 DESCRIPTION

The I<disks_gui.pl> displays a GUI which lets you view and update
the disk database.

=head1 FILES

=over 4

=item disks.db

The name of the database file.  The actual path to this file is 
controlled by the I<disks_config.pm> package.

=back

=head1 AUTHOR

Steve Oualline, E<lt>oualline@www.oualline.comE<gt>.

=head1 COPYRIGHT

This program is distributed under the GPL.

=cut

use strict;
use warnings;
use Tk;
use Tk::LabEntry;

use disks;
use disks_config;

my $cur_category = "All";# Current category
my $tk_mw;      # Main window
my $tk_list;    # The list of disks
my $tk_find_next;# Button to find then next item

my @id_list = ();       # A list of ID's displayed in the
                        # GUI list

my %reverse_category;        # Reverse category list
####################################################
# error($message)
#
# Display a popup error message window
####################################################
sub error($)
{
    my $message = shift;	 # Error message

    $tk_mw->messageBox(
        -icon => "error",
        -title => "Error",
        -message => $message,
        -type => "ok");
}
####################################################
# info($message)
#
# Display a popup information message window
####################################################
sub info($)
{
    my $message = shift;	 # Error message

    $tk_mw->messageBox(
        -icon => "info",
        -title => "Information",
        -message => $message,
        -type => "ok");
}
########################################################
# Populate list -- Put the disks in the list
########################################################
sub populate_list()
{
    # The database containing the disk informatin
    my $db = disks->new($db_file, 1);

    $tk_list->delete(0, 'end');

    if (not defined($db)) {
	return;
    }
    # The data in the database
    my @key_list = $db->get_keys();

    my $cat_letter;     # Category as letter

    if ($cur_category ne "All") {
        $cat_letter = $reverse_category{$cur_category};
    } else {
        $cat_letter = "";
    }
    @id_list = ();

    # Print each as we find it
    foreach my $cur_id (@key_list) {
        # The data from the database
        my $data = $db->get_disk($cur_id);

        # Filter on category
        if (($cat_letter ne "") and 
            ($cat_letter ne $data->{CATEGORY})) {
            next;
        }

        # The data as a line
        my $line = sprintf "%3s: %-s", $cur_id, $data->{DESCRIPTION};
        $tk_list->insert('end', $line);
        push(@id_list, $cur_id);
    }
}
########################################################
# change_option -- Called when we change the category
########################################################
sub change_option {
    if (defined ($tk_list)) {
	populate_list();
    }
}
########################################################
# remove_disk -- Remove a disk from the list
########################################################
sub remove_disk {
    my $selection = $tk_list->curselection();
    if (not defined($selection)) {
        error("No disk selected.");
        return;
    }
    my $db = disks->new($db_file, 0);
    $db->delete_disk($id_list[$selection]);
    $db = undef;
    populate_list();
}

my $description = "";	# Description for add/modify
my $add_category = "";	# Category for the current add/modify
my $tk_popup;	# Popup menu for add/modify
########################################################
# do_add -- Actually add the disk to the system
########################################################
sub do_add()
{
    my $db = disks->new($db_file, 0);
    if ($description =~ /^\s*$/) {
        error("ERROR: No description");
        return;
    }
    $tk_popup->withdraw();
    my $id = $db->new_disk(
            $reverse_category{$add_category},
            CATEGORY => $reverse_category{$add_category},
            DESCRIPTION => $description
        );
    $db = undef;

    populate_list();

    if (defined($id)) {
        info("Added $id");
    } else {
        error("Could not add $id");
    }
}
########################################################
# do_modify -- Actually modify the information
########################################################
sub do_modify($)
{
    my $id = shift;
    my $db = disks->new($db_file, 0);
    if ($description =~ /^\s*$/) {
        error("No description.");
        return;
    }
    $tk_popup->withdraw();

    my $cat = substr($id, 0, 1);
    $db->update_disk($id,
            CATEGORY => $cat,
            DESCRIPTION => $description
        );
    $db = undef;

    populate_list();
    info("Updated $id");
}
########################################################
# add_modify($id) -- Do an add or modify
# 
# If $id is defined, modify it.  Otherwise add it
########################################################
sub add_modify($) {
    my $id = shift;     # Id of the disk to modify
                        # undef if no such disk

    # Remove any old popup window
    if (Exists($tk_popup)) {
        $tk_popup->destroy();
    }
    # Top level window
    $tk_popup = $tk_mw->Toplevel();

    # List of catigories for the iption
    my @options = (sort values %category_to_name);

    if (defined($id)) {
        # The database for the information
        my $db = disks->new($db_file, 1);

        # Data for the disk
        my $data = $db->get_disk($id);

        $description = $data->{DESCRIPTION};
        $tk_popup->Label(
                -text => $category_to_name{$data->{CATEGORY}}
            )->pack(
                -side => 'top'
            );
    } else {
        $description = "";

        $tk_popup->Optionmenu(
                -options => [@options],
                -variable => \$add_category
            )->pack(
                -side => 'top'
            );
    }
   
    $tk_popup->LabEntry(
            -label => "Description",
            -textvariable => \$description,
	    -labelPack => [ -side => 'left']
        )->pack(
            -side => 'top',
            -expand => 1,
            -fill => 'x'
        );

    my $tk_action = $tk_popup->Frame(
            )->pack(
                -side => 'bottom',
                -expand => 1,
                -fill => 'x'
            );

    if (defined($id)) {
        $tk_action->Button(
                -text => "Modify",
                -command => sub {do_modify($id);}
            )->pack(
                -side => 'left'
            );
    } else {
        $tk_action->Button(
                -text => "Add",
                -command => \&do_add
            )->pack(
                -side => 'left'
            );
    }
    $tk_action->Button(
            -text => "Cancel",
            -command => sub {$tk_popup->destroy();}
        )->pack(
            -side => 'right'
        );
}

########################################################
# add_disk -- Add a disk to the set
########################################################
sub add_disk {
    add_modify(undef);
}
########################################################
# modify_disk -- Modify a disk
########################################################
sub modify_disk {
    my $selection = $tk_list->curselection();
    if (not defined($selection)) {
        error("No disk selected.");
        return;
    }
    add_modify($id_list[$selection]);
}
my $tk_find;	# Find dialog
my $find_str;	# String to find
########################################################
# do_find -- Find the next entry that matches
#	the find string
########################################################
sub do_find($) {
    my $start = shift;	# Element to start searching at

    $tk_find_next->configure(-state => 'disabled');
    if ($find_str ne "") {
	$tk_find_next->configure(-state => 'normal');
    }
    for (my $current = $start;
         $current < $tk_list->index('end');
	 $current++) {

	# Check the line against what are looking at
        my $line = $tk_list->get($current);
	if ($line =~ $find_str) {
	    $tk_list->selectionClear(0, 'end');
	    $tk_list->selectionSet($current);
	    $tk_list->see($current);
	    return;
	}
    }
    error("Not found");
}

########################################################
# find_next -- find the next string in the list
########################################################
sub find_next()
{
    do_find($tk_list->curselection()+1);
}
########################################################
# find -- Find the first element that matches a patter
########################################################
sub find()
{
    # Remove any old popup window
    if (Exists($tk_find)) {
        $tk_find->destroy();
    }
    # Top level window
    $tk_find = $tk_mw->Toplevel();

    $tk_find->LabEntry(
            -label => "Find: ",
            -textvariable => \$find_str,
	    -labelPack => [ -side => 'left']
        )->pack(
            -side => 'top',
            -expand => 1,
            -fill => 'x'
        );

    my $tk_action = $tk_find->Frame(
            )->pack(
                -side => 'bottom',
                -expand => 1,
                -fill => 'x'
            );

    $tk_action->Button(
	    -text => "Find",
	    -command => sub {$tk_find->withdraw();do_find(0);}
	)->pack(
	    -side => 'left'
	);
    $tk_action->Button(
            -text => "Cancel",
            -command => sub {$tk_find->destroy();}
        )->pack(
            -side => 'right'
        );
}

########################################################
# Main program
########################################################
$tk_mw = MainWindow->new();

# The list of the types of disks we can have
my @options = ((sort values %category_to_name), "All");

# Category name to key translation
%reverse_category = reverse %category_to_name;

# Define the type selector at the top of the screen
my $tk_option = $tk_mw->Optionmenu(
        -options => [@options],
        -command => \&change_option,
        -variable => \$cur_category
    )->pack(
        -side => 'top'
    );

$tk_option->setOption("All");

$tk_list = $tk_mw->Scrolled(
        'Listbox',
        -scrollbars => "ose",
	-width => 50
    )->pack(
        -side => 'top',
        -expand => 'yes',
        -fill => 'both'
    );
populate_list();

# The frame containing the action buttons at the bottom
# of the screen
my $tk_action_frame = $tk_mw->Frame(
    )->pack(
        -side => 'bottom',
        -fill => "x"
    );

$tk_action_frame->Button(
        -text => "Add",
        -command => \&add_disk
    )->pack( 
        -side => 'left'
    );

$tk_action_frame->Button(
        -text => "Remove",
        -command => \&remove_disk
    )->pack( 
        -side => 'left'
    );

$tk_action_frame->Button(
        -text => "Modify",
        -command => \&modify_disk
    )->pack( 
        -side => 'left'
    );

$tk_action_frame->Button(
        -text => "Find",
        -command => \&find
    )->pack( 
        -side => 'left'
    );

$tk_find_next = $tk_action_frame->Button(
        -text => "Find Next",
        -command => \&find_next,
	-state => 'disabled'
    )->pack( 
        -side => 'left'
    );


$tk_action_frame->Button(
        -text => "Exit",
        -command => sub {exit(0);}
    )->pack( 
        -side => 'right'
    );
MainLoop();
